@@ -5,3 +5,5 @@ cur_serial_port = -1
 cur_iter = -1
 cur_font = -1
 build_verbose = -1
+user_library = -1
+serial_baud_rate = -1
